#!/usr/bin/python
import math
import pdb
from OpenGL.GL import *
from OpenGL.GLU import *
from PyQt4 import QtGui
from PyQt4.QtOpenGL import *
from PyQt4.Qt import Qt
from PyQt4 import QtCore
from PyNGL import *
import sys
import random

class GLWindow(QGLWidget):

	def __init__(self, parent):
		QGLWidget.__init__(self, parent)
		# enumeration for move modes
		self.EYE=0
		self.LOOK=1
		self.BOTH=2
		self.setMinimumSize(1024, 720)
		self.m_spinYFace = 0
		self.m_spinXFace = 0
		self.m_origX = 0
		self.m_origY = 0
		self.m_transformStack=TransformStack()

		self.m_cameras=[]
		self.m_cameraIndex=0
		self.m_moveMode=self.EYE
		self.m_drawHelp=True
		self.m_fov=65
		self.m_aspect=float(1024/720)
		self.m_projection=CAMERAPROJECTION.PERSPECTIVE
		self.m_rotation=0.0
		self.m_lights=[True,True,True]

	def mousePressEvent (self,  _event) :

		# this method is called when the mouse button is pressed in this case we
		# store the value where the maouse was clicked (x,y) and set the Rotate flag to true
		if(_event.button() == Qt.LeftButton) :
			self.m_origX = _event.x()
			self.m_origY = _event.y()
			self.m_rotate =True



	def mouseMoveEvent ( self,_event ) :
		# note the method buttons() is the button state when event was called
		# this is different from button() which is used to check which button was
		# pressed when the mousePress/Release event is generated
		if(self.m_rotate and _event.buttons() == Qt.LeftButton) :
			self.m_spinYFace = ( self.m_spinYFace + (_event.x() - self.m_origX) ) % 360
			self.m_spinXFace = ( self.m_spinXFace + (_event.y() - self.m_origY) ) % 360
			self.m_origX = _event.x()
			self.m_origY = _event.y()
			# re-draw GL
			self.updateGL()

	def mouseReleaseEvent (self,  _event) :

		# this event is called when the mouse button is released
		# we then set Rotate to false
		if (_event.button() == Qt.LeftButton) :
			self.m_rotate=False


	def paintGL(self):
			'''
			Drawing routine
			'''
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)


			trans=Transformation()
			# set the mouse rotation
			trans.setRotation(self.m_spinXFace,self.m_spinYFace,0)
			# set this in the TX stack
			self.m_transformStack.setGlobal(trans)

			shader=ShaderManager.instance()
			shader.setShaderParamFromMatrix("gl3xTest","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getTransposeMatrix())
			shader.useShader("gl3xTest")
			shader.setShaderParamFromMatrix("gl3xTest","ViewMatrix",self.m_cameras[self.m_cameraIndex].getModelView())
			shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())
			shader.setShaderParam4f("gl3xTest","eye",self.m_cameras[self.m_cameraIndex].getEye()[0],self.m_cameras[self.m_cameraIndex].getEye()[1],self.m_cameras[self.m_cameraIndex].getEye()[2],1.0)



			prim=VBOPrimitives.instance()


			L1=Light(Vector(-1,1,0),Colour(1,1,1,1),LIGHTMODES.LIGHTREMOTE)
			L2=Light(Vector(0,1,0),Colour(0.5,0.5,0.5,1),LIGHTMODES.LIGHTLOCAL)
			L3=Light(Vector(1,1,0),Colour(1,1,1,1),LIGHTMODES.LIGHTREMOTE)
			L1.setSpecColour(Colour(0.5,0.5,0.5))
			L2.setSpecColour(Colour(0.5,0.5,0.5))
			L3.setSpecColour(Colour(0.5,0.5,0.5))
			numEnabledLights=0
			if self.m_lights[0] == True :
				L1.enable()
				numEnabledLights+=1
			if self.m_lights[1] == True :
				L2.enable()
				numEnabledLights+=1
			if self.m_lights[2] == True :
				L3.enable()
				numEnabledLights+=1



			shader.setShaderParam1i("gl3xTest","numLightsEnabled",numEnabledLights)


			self.m_transformStack.getCurrentTransform().setRotation(0,self.m_rotation,0)

			self.shader.setShaderParamFromMatrix("gl3xTest","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getMatrix())

			prim.draw("teapot")
			# now we are going to draw the grid in wireframe mode
			# load the colour  shader and set the values,
			self.shader.useShader("Colour")
			# at the outer level of the drawing we set the view and projection
			self.shader.setShaderParamFromMatrix("Colour","ViewMatrix",self.m_cameras[self.m_cameraIndex].getModelView())
			self.shader.setShaderParamFromMatrix("Colour","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())
			self.m_transformStack.pushTransform()
			# translate the grid down by -0.5 so it is at the base level of the objects
			self.m_transformStack.getCurrentTransform().setPosition(0,-0.5,0)
			# now we pass this modelling transform to the shader ModelMatrix
			self.shader.setShaderParamFromMatrix("Colour","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getMatrix())
			# finally draw the plane
			#glPolygonMode(GL_FRONT_AND_BACK,GL_LINE)

			prim.draw("plane")

			self.m_transformStack.popTransform()
			# now go back to solid drawing
			#glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
			# activate the shader
			self.shader.useShader("gl3xTest")
			# push a transform
			self.m_transformStack.pushTransform()

			# translate into position
			self.m_transformStack.getCurrentTransform().setPosition(-1.8,0.1,0)
			# rotate object
			self.m_transformStack.getCurrentTransform().setRotation(self.m_rotation,self.m_rotation,self.m_rotation)
			# pass values to the shader
			self.shader.setShaderParamFromMatrix("gl3xTest","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getMatrix())
			# draw
			prim.draw("cube")

			self.m_transformStack.popTransform()
			# now we draw the football
			self.m_transformStack.pushTransform()

			# set the transforms for pos and scale
			self.m_transformStack.getCurrentTransform().setPosition(1.5,0,0)
			self.m_transformStack.getCurrentTransform().setScale(0.5,0.5,0.5)
			self.m_transformStack.getCurrentTransform().setRotation(self.m_rotation,self.m_rotation,self.m_rotation)

			# pass to the shader
			self.shader.setShaderParamFromMatrix("gl3xTest","ModelMatrix",self.m_transformStack.getCurrAndGlobal().getMatrix())
			# draw the football
			prim.draw("football")

			self.m_transformStack.popTransform()


			glFlush()



	def resizeGL(self, _w, _h):
			'''
			Resize the GL window
			'''
			glViewport(0, 0, _w, _h)

			# calculate the aspect ratio
			self.m_aspect=float(_w/_h)
			# now set the camera values
			self.setCameraShape()

	def setCameraShape(self) :
		for i in range(0,len(self.m_cameras)) :
			self.m_cameras[i].setShape(self.m_fov,self.m_aspect, 0.2,150.0,self.m_projection)
			self.m_cameras[i].setOrthoParams(-1,1,-1,1,1,10.0)

		self.shader.useShader("Colour")
		self.shader.setShaderParamFromMatrix("Colour","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())
		self.shader.useShader("gl3xTest")
		self.shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())



	def createCameras(self) :
		EYE=Vector(0,5,8,1)
		LOOK=Vector(0.0,0.0,0.0,0)
		UP=Vector(0,1,0,1)

		TEYE=Vector(0,8,0)
		TLOOK=Vector(0.0,0.0,0.0,0)
		TUP=Vector(0,0,1)

		SEYE=Vector(8,0,0)
		SLOOK=Vector(0.0,0.0,0.0,0)
		SUP=Vector(0,1,0)

		FEYE=Vector(5,2,8)
		FLOOK=Vector(0.0,0.0,0.0)
		FUP=Vector(0,1,0)
		# finally set the cameras shape and position
		cam=Camera()
		cam.set(EYE,LOOK,UP)
		cam.setShape(self.m_fov,self.m_aspect, 0.2,150.0,self.m_projection)
		cam.setOrthoParams(-1,1,-1,1,0.1,10.0)
		self.m_cameras.append(cam)
		Tcam=Camera()
		Tcam.set(TEYE,TLOOK,TUP)
		Tcam.setShape(self.m_fov,self.m_aspect, 0.5,150.0,self.m_projection)
		Tcam.setOrthoParams(-0.1,0.1,-0.1,0.1,0.1,10.0)
		self.m_cameras.append(Tcam)
		Scam=Camera()
		Scam.set(SEYE,SLOOK,SUP)
		Scam.setShape(self.m_fov,self.m_aspect, 0.5,150.0,self.m_projection)
		Scam.setOrthoParams(-0.1,0.1,-0.1,0.1,0.1,10.0)
		self.m_cameras.append(Scam)
		Fcam=Camera()
		Fcam.set(FEYE,FLOOK,FUP)
		Fcam.setShape(self.m_fov,self.m_aspect, 0.5,150.0,self.m_projection)
		Fcam.setOrthoParams(-0.1,0.1,-0.1,0.1,0.1,10.0)
		self.m_cameras.append(Fcam)

	def initializeGL(self):
			'''
			Initialize GL
			'''
			# set viewing projection
			ngl=NGLInit.instance()
			ngl.initGlew()
			glClearColor(0.4, 0.4, 0.4, 1.0)
			glClearDepth(1.0)
			self.createCameras()
			glEnable(GL_DEPTH_TEST)
			prim=VBOPrimitives.instance()
			prim.createVBOQuadPlane("plane",20,20,30,30,Vector(0,1,0))
			prim.createVBOSphere("sphere",1.0,150)

			self.shader=ShaderManager.instance()

			self.shader.loadShader("Colour","Colour.vs","Colour.fs","",False)
			# set this as the active shader
			self.shader.useShader("Colour")
			self.shader.setShaderParamFromMatrix("Colour","ViewMatrix",self.m_cameras[self.m_cameraIndex].getModelView())
			self.shader.setShaderParamFromMatrix("Colour","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())


			# load a frag and vert shaders
			self.shader.loadShader("gl3xTest","Vertex.vs","Fragment.fs","",True)
			# set this as the active shader
			self.shader.useShader("gl3xTest")
			# now pass the modelView and projection values to the shader
			self.shader.setShaderParamFromMatrix("gl3xTest","ViewMatrix",self.m_cameras[self.m_cameraIndex].getModelView())
			self.shader.setShaderParamFromMatrix("gl3xTest","projectionMatrix",self.m_cameras[self.m_cameraIndex].getProjection())
			self.shader.setShaderParam1i("gl3xTest","Normalize",True)
			self.shader.setShaderParam1i("gl3xTest","numLightsEnabled",3)
			self.shader.setShaderParam3f("gl3xTest","eye",self.m_cameras[self.m_cameraIndex].getEye()[0],self.m_cameras[self.m_cameraIndex].getEye()[1],self.m_cameras[self.m_cameraIndex].getEye()[2])
			m=Material(STDMAT.GOLD)
			m.use()

# You don't need anything below this
class NGLOpenGLDemo(QtGui.QMainWindow):

	def __init__(self):
		QtGui.QMainWindow.__init__(self)
		self.widget = GLWindow(self)
		self.setCentralWidget(self.widget)


	def keyPressEvent(self ,_event) :
		INC=0.1

		if _event.key() == Qt.Key_Q	:
			sys.exit()
		elif _event.key() == Qt.Key_W :
			glPolygonMode(GL_FRONT_AND_BACK,GL_LINE)
		elif _event.key() == Qt.Key_S :
			glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)
		elif _event.key() == Qt.Key_Up :
			self.widget.m_cam.Move(0.0,-INC,0.0)
		elif _event.key() == Qt.Key_Down :
			self.widget.m_cam.Move(0.0,INC,0.0)
		elif _event.key() == Qt.Key_Left :
			self.widget.m_cam.Move(INC,0.0,0.0)
		elif _event.key() == Qt.Key_Right :
			self.widget.m_cam.Move(-INC,0.0,0.0)

		elif _event.key() == Qt.Key_I :
			self.widget.m_cam.Move(0.0,0.0,INC)
		elif _event.key() == Qt.Key_O :
			self.widget.m_cam.Move(0.0,0.0,-INC)

		self.widget.updateGL()


if __name__ == '__main__':
	app = QtGui.QApplication(['Simple NGL Python Demo'])
	window = NGLOpenGLDemo()
	window.setWindowTitle("PyNGL Demo")
	window.show()
	app.exec_()